// Copyright (c) 2002,2003 Jesper Pedersen, David Greaves
//
//     This file is part of Shopper.

//     Shopper is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     Shopper is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with Shopper; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//     $Id: itemconfig.cpp,v 1.2 2003/12/11 16:16:37 arbor Exp $

#include "Shopper.h"
#include "itemconfig.h"
#include <qhbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qaccel.h>

ItemConfig::ItemConfig( QWidget* parent ) : QWidget( parent, "ItemConfig" ) 
{
  QVBoxLayout* lay1 = new QVBoxLayout( this, 6 );
  QGridLayout* lay2 = new QGridLayout( lay1, 2, 2, 6 );
  QHBoxLayout* lay3 = new QHBoxLayout( lay1, 6 );

  QLabel* label = new QLabel( tr("Name"), this );
  _name = new QLineEdit( this );
  lay2->addWidget( label, 0, 0 );
  lay2->addWidget( _name, 0, 1 );
  
  label = new QLabel( tr("Category"), this );
  _category = new QComboBox( this );
  lay2->addWidget( label, 1, 0 );
  lay2->addWidget( _category, 1, 1 );

  _ok = new QPushButton( tr("OK"), this );
  _del = new QPushButton( tr("Delete"), this );
  QPushButton* cancel = new QPushButton( tr("Cancel"), this );
  lay3->addStretch(1);
  lay3->addWidget( _ok );
  lay3->addWidget( _del );
  lay3->addWidget( cancel );
  
  connect( cancel, SIGNAL( clicked() ), this, SIGNAL( closeMe() ) );
  connect( _del, SIGNAL( clicked() ), this, SIGNAL( closeMe() ) );
  connect( _del, SIGNAL( clicked() ), this, SIGNAL( deleteItem() ) );
  connect( _name, SIGNAL( textChanged( const QString& ) ), this, SLOT( nameChanged( const QString& ) ) );
  connect( _ok, SIGNAL( clicked() ), this, SLOT( done() ) );
  connect( _name, SIGNAL( returnPressed() ), this, SLOT( done() ) );
  
  QAccel* accel = new QAccel(this);
  int id = accel->insertItem( Key_Return );
  accel->connectItem( id, this, SLOT( done() ) );
  
  lay1->addStretch(1);
}

void ItemConfig::done() 
{  
  int index = _category->currentItem();
  CategoryItem* item = _categories->at(index);
  emit selected( _name->text(), item );
  emit closeMe();
}

void ItemConfig::setDeletePossible( bool on ) 
{
  _del->setEnabled( on );
}

void ItemConfig::setName( const QString& name )
{
  _name->setText( name );
}

void ItemConfig::setCategory( CategoryItem* category, CategoryList* available )
{
  _category->clear();
  int index = 0;
  
  for ( CategoryListIterator it( *available ); *it; ++it, ++index ) {
    _category->insertItem( (*it)->text() );
    if ( *it == category ) {
      _category->setCurrentItem( index );
    }
  }
  _categories = available;
}

void ItemConfig::nameChanged( const QString& str )
{
  _ok->setEnabled( !str.isEmpty() );
}


