// Copyright (c) 2003 David Greaves
//
//     This file is part of Shopper.

//     Shopper is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     Shopper is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with Shopper; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//     $Id: prefsdialog.cpp,v 1.2 2004/04/10 20:55:18 arbor Exp $

#include "Shopper.h"
#include "prefsdialog.h"

#include <qcheckbox.h>
#include <qlayout.h>
#include <qvariant.h>

/* 
 *  Constructs a prefsDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
PrefsDialog::PrefsDialog(  Options *options, QWidget* parent,  const char* name, WFlags fl)
  : QTabDialog( parent, name, fl ), _options(options)
{
  setEnabled( TRUE );
  setMinimumHeight(340);

  _tab = new QWidget( this, "tab" );

  _cShowNotes = new QCheckBox( _tab, "cShowNotes" );
  _cShowNotes->setGeometry( QRect( 10, 10, 140, 20 ) ); 
  _cShowNotes->setText( tr( "Show notes area" ) );

  _cShowCategory = new QCheckBox( _tab, "cShowCategory" );
  _cShowCategory->setGeometry( QRect( 10, 40, 140, 20 ) ); 
  _cShowCategory->setText( tr( "Show category in list" ) );

  // Don't show this yet...
  _cShowCategory->hide();

  _cFastChange = new QCheckBox( _tab, "cFastChange" );
  _cFastChange->setGeometry( QRect( 10, 70, 200, 20 ) ); 
  _cFastChange->setText( "Speed up category menu\n(may flicker)");

  insertTab( _tab, tr( "List Display" ) );
  setCancelButton("Cancel");

  connect (this, SIGNAL(aboutToShow()), this, SLOT(getPrefs()));
  connect (this, SIGNAL(applyButtonPressed()), this, SLOT(setPrefs()));
  connect (this, SIGNAL(cancelButtonPressed()), SIGNAL(done()));

//  tab_2 = new QWidget( this, "tab_2" );
//  prefsDialog->insertTab( tab_2, tr( "Tab 2" ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
PrefsDialog::~PrefsDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

void PrefsDialog::getPrefs()
{
  _cShowNotes->setChecked(_options->getOption("show.notes") == "on");
  _cShowCategory->setChecked(_options->getOption("show.category") == "on");
  _cFastChange->setChecked(_options->getOption("fast.change") == "on");
}
void PrefsDialog::setPrefs()
{
  _options->setOption("show.category", _cShowCategory->isOn()?"on":"off");
  _options->setOption("show.notes", _cShowNotes->isOn()?"on":"off");
  _options->setOption("fast.change", _cFastChange->isOn()?"on":"off");
  emit(done());
}

