// Copyright (c) 2002,2003 Jesper Pedersen, David Greaves
//
//     This file is part of Shopper.

//     Shopper is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     Shopper is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with Shopper; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//     $Id: shoplist.cpp,v 1.4 2004/04/10 20:55:18 arbor Exp $

#include "Shopper.h"
#include "shoplist.h"
#include <qpopupmenu.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qscrollview.h>
#include <qfile.h>
#include <qpe/global.h>
#include <qxml.h>
#include <qheader.h>
#include <qevent.h>
#include "categoryitem.h"
#include "categoryitem.h"
#include "prefsdialog.h"
#include "options.h"
#include <qtextstream.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qregexp.h>


// TODO: Somewhere it should say:
//  if (mode == ShoppingItem::Left && nothing_left) {
//    QMessageBox::information(this, tr( "Go Home!" ), tr ("<qt>You're done - you've got everything on the list :)</qt>" ));
//  }


ShopList::ShopList( QWidget* parent, const char* name )
  :QMainWindow( parent, name ), _showAll( true )
{
  QMenuBar* bar = menuBar();
  _bar=bar;

  // Setup menu (Add a new item / reset items)
  QPopupMenu* itemMenu = new QPopupMenu( this );
  bar->insertItem( tr("Setup"), itemMenu, 1 );

  QAction* addItem = new QAction( "", QIconSet(), tr("Add Item"), 0, this );
  addItem->addTo( itemMenu );
  connect( addItem, SIGNAL( activated() ), this, SLOT( addNewItem() ) );
  QAction* clearItems = new QAction( "", QIconSet(), tr("New Shopping Trip"), 0, this );
  clearItems->addTo( itemMenu );
  connect( clearItems, SIGNAL( activated() ), this, SLOT( resetItems() ) );
  QAction* setPrefs = new QAction( "", QIconSet(), tr("Preferences"), 0, this );
  setPrefs->addTo( itemMenu );
  connect( setPrefs, SIGNAL( activated() ), this, SLOT( setPreferences() ) );

  // Mode menu (Making a list / Shopping)
  QPopupMenu* doingMenu = new QPopupMenu( this );
  bar->insertItem( tr("Mode"), doingMenu, 1 );

  QActionGroup* group = new QActionGroup(this);
  group->setExclusive( true );

  _viewLeftMode = new QAction( "", QIconSet(), tr("Out Shopping: What's left?"), 0, this );
  _viewLeftMode->addTo( doingMenu );
  connect( _viewLeftMode, SIGNAL( activated() ), this, SLOT( showLeftItems() ) );
  _viewLeftMode->setToggleAction( true );
  group->insert( _viewLeftMode );

  _viewNeededMode = new QAction( "", QIconSet(), tr("Out Shopping: Full list"), 0, this );
  _viewNeededMode->addTo( doingMenu );
  connect( _viewNeededMode, SIGNAL( activated() ), this, SLOT( showNeededItems() ) );
  _viewNeededMode->setToggleAction( true );
  group->insert( _viewNeededMode );

  _viewAllMode = new QAction( "", QIconSet(), tr("Making a List"), 0, this );
  _viewAllMode->addTo( doingMenu );
  connect( _viewAllMode, SIGNAL( activated() ), this, SLOT( showAllItems() ) );
  _viewAllMode->setToggleAction( true );
  group->insert( _viewAllMode );
  _viewAllMode->setOn( true );

  // 'Prev Category' menu item (<)
  bar->connectItem(bar->insertItem( tr("<"), -1,-1),
		   this, SLOT( showPrevCat() ) );

  // Category menu (auto generated as it's viewed)
  _categoryMenu = new QPopupMenu( this );
  _currCategoryPos = bar->insertItem( tr("Category: Everything"), _categoryMenu );
  connect( _categoryMenu, SIGNAL( aboutToShow() ), this, SLOT( populateCategories() ) );

  // I'd hoped this would align the ">" to the right... but no :(
  bar->insertSeparator();

  // 'Next Category' menu item (>)
  bar->connectItem(bar->insertItem( tr(">"), -1,-1),
		   this, SLOT( showNextCat() ) );

  // View
  _stack = new QWidgetStack( this );
  setCentralWidget( _stack );

  _view = new QScrollView( _stack );
  _view->setResizePolicy( QScrollView::AutoOneFit );
  _stack->addWidget( _view, -1 );
  _itemView = new QWidget( _view->viewport() );
  _view->addChild( _itemView );
  _viewManager = 0;

  // Item Config box
  _itemConfig = new ItemConfig( _stack );
  _stack->addWidget( _itemConfig, -1 );
  connect( _itemConfig, SIGNAL( closeMe() ), this, SLOT( raiseView() ) );
  connect( _itemConfig, SIGNAL( selected( const QString&, CategoryItem* ) ),
           this, SLOT( editItemFinished(  const QString&, CategoryItem* ) ) );
  connect( _itemConfig, SIGNAL( deleteItem() ), this, SLOT( deleteCurrentItem() ) );

  // Category Config box
  _categoryConfig = new CategoryConfig( _stack );
  _stack->addWidget( _categoryConfig, -1 );
  connect( _categoryConfig, SIGNAL( closeMe() ), this, SLOT( raiseView() ) );
  connect( _categoryConfig, SIGNAL( selected( const QString& ) ),
           this, SLOT( editCategoryFinished(  const QString& ) ) );
  connect( _categoryConfig, SIGNAL( deleteItem() ), this, SLOT( deleteCurrentCategory() ) );

  // Category Actions (for use by populateCategories() )
  _categoryGroup = new QActionGroup(this);
  connect( _categoryGroup, SIGNAL( selected( QAction* ) ), this, SLOT( changeCategory( QAction* ) ) );

  _addCategoryAction = new QAction( "", QIconSet(), tr("Add Category"), 0, this );
  connect( _addCategoryAction, SIGNAL( activated() ), this, SLOT( addCategory() ) );

  _editCategoryAction = new QAction( "", QIconSet(), tr("Edit Current"), 0, this );
  connect( _editCategoryAction, SIGNAL( activated() ), this, SLOT( editCategory() ) );

  _showAllAction = new QAction( "", QIconSet(), tr("Everything"), 0, this );
  _showAllAction->setToggleAction( true );
  _showAllAction->setOn( _showAll );
  _categoryGroup->insert( _showAllAction );
  _stack->raiseWidget( _view );

  // Preferences Dialog
  _prefsDialog = new PrefsDialog( &_options, _stack );
  _stack->addWidget( _prefsDialog, -1 );
  connect( _prefsDialog, SIGNAL( done() ), this, SLOT( raiseView() ) );


  // Read in the saved state and update the view.
  load();
  // Only show the Item menu if there are any categories.
  if ( _categories.isEmpty() ) {
    menuBar()->setItemEnabled( 1, false ); // Setup Menu
    _editCategoryAction->setEnabled( false );
  }
}

//######################################################################
// XML

// Writes the shoppinglist state in XML format
// Includes: viewmode (Making/Shopping:full/Shopping:left)
//           Categories (inc current)
//           Items (and needed/bought state)
void ShopList::save()
{
  QString fileName = Global::applicationFileName("ShoppingListQt", "shoppinglist.xml");
  QString name, note;
  QFile file( fileName );
  if ( ! file.open( IO_WriteOnly ) ) {
    qDebug("Ups!");
  }
  else {
    QTextStream stream( &file );
    QString res = QString("<ShoppingList mode=\"%1\">\n").arg(_viewMode);
    for ( OptionsIt ito = _options.begin(); ito.hasNext() ; ++ito ) {
      res += QString("\t<Option name=\"%1\" value=\"%2\" />\n").
	arg(*ito).arg(_options.getOption(*ito));
    }
    for ( QListIterator<CategoryItem> it( _categories ); *it ; ++it ) {
      name=(*it)->text();
      escape(&name);
      res += QString("\t<Category name=\"" + name + "\" current=\"%1\">\n")
	.arg(_currentCategory == *it);
      for ( ItemListIterator it2( _items ); *it2; ++it2 ) {
	name=(*it2)->text();
	escape(&name);
        if ( (*it2)->category() == *it ) {
	  note=(*it2)->note();
	  escape(&note);
          res += QString("\t\t<Item name=\"%1\" needed=\"%2\" bought=\"%3\" note=\"%4\" />\n")
            .arg(name).arg((*it2)->isNeeded()).arg((*it2)->isBought()).arg(note) ;
        }
      }
      res += "\t</Category>\n";
    }
    res += "</ShoppingList>\n";
    stream << res.local8Bit();
    file.close();
  }
}

// 'escapes' the XML to make it safe (eg "Cross & Blackwell"...)
void ShopList::escape(QString* s)
{
  s->replace(QRegExp("&"), "&amp;"); // This _must_ come first
  s->replace(QRegExp("<"), "&lt;");
  s->replace(QRegExp(">"), "&gt;");
  s->replace(QRegExp("\""), "&quot;");
  s->replace(QRegExp("'"), "&apos;");
}

// Loads the shoppinglist state in XML format
// Includes: viewmode (Making/Shopping:full/Shopping:left)
//           Categories
//           Items (and needed/bought state)
void ShopList::load()
{
  QString fileName = Global::applicationFileName("ShoppingListQt", "shoppinglist.xml");

  QFile file( fileName );
  _startCategory = static_cast<CategoryItem*>(_showAllAction);
  // If there's a file - read it. If not then we drop through to do
  // other initialisation.
  if ( file.exists() ) {
    QXmlInputSource source( file );
    QXmlSimpleReader reader;
    reader.setContentHandler( this );
    reader.parse( source );
  }
  _items.sort();
  if (_startMode == ShoppingItem::Needed)
    showNeededItems();
  else if (_startMode == ShoppingItem::Left)
    showLeftItems();
  else
    showAllItems();
  changeCategory( _startCategory );
}

// Consider each XML element in the list
bool ShopList::startElement( const QString&, const QString&,
                             const QString& qName,
                             const QXmlAttributes& attributes)
{
  if ( qName == "ShoppingList" ) {
    if (attributes.value("mode") == "1") _startMode = ShoppingItem::Needed;
    else if (attributes.value("mode") == "2") _startMode = ShoppingItem::Left;
    else _startMode = ShoppingItem::Making;
  }

  else if ( qName == "Option" ) {
    QString option = attributes.value("name");
    QString value = attributes.value("value");
    _options.setOption(option, value);
  }

  else if ( qName == "Category" ) {
    QString category = attributes.value("name");
    _myCategory = addCategory( category );
    if (attributes.value("current") == "1") {
      _startCategory = _myCategory;
    }
  }

  else if ( qName == "Item" ) {
    addItem( attributes.value("name"), _myCategory,
             attributes.value("needed") == "1",
             attributes.value("bought") == "1",
	     attributes.value("note"));
  }
  else {
    qDebug("Unknown element %s", qName.latin1());
  }

  return TRUE;
}

// Used in loading XML
void ShopList::addItem( const QString& name, CategoryItem* category,
			bool needed, bool bought, const QString& note )
{
    ShoppingItem* item = new ShoppingItem( name, category, needed, bought, note, &_options, _itemView );
    connect( item, SIGNAL( edit( ShoppingItem* ) ), this, SLOT( editItem( ShoppingItem* ) ) );
    _items.append( item );
}

// Used to add a category non-interactivley (eg from XML and category dialog)
CategoryItem* ShopList::addCategory( const QString& name )
{
  for ( QListIterator<CategoryItem> it( _categories ); *it ; ++it ) {
    if ( (*it)->text() == name ) {
      return *it;
    }
  }

  CategoryItem* item = new CategoryItem( name, this );
  item->setToggleAction( true );
  _categories.inSort( item );
  _categoryGroup->insert( item );
  menuBar()->setItemEnabled( 1, true ); // Setup Menu
  _editCategoryAction->setEnabled( true );
  return item;
}

//######################################################################
// GUI

void ShopList::raiseView()
{
  _stack->raiseWidget( _view );
}

void ShopList::closeEvent( QCloseEvent* event )
{
  if ( _stack->visibleWidget() != _view ) {
    _stack->raiseWidget( _view );
    event->ignore();
  }
  else {
    save();
    QMainWindow::closeEvent( event );
  }
}

// A 'slot' used by the Preferences menu
void ShopList::setPreferences()
{
  _stack->raiseWidget( _prefsDialog );
}

// A 'slot' used by the AddItem menu
void ShopList::addNewItem()
{
  _currentEdit = 0;
  _itemConfig->setName( "" );
  _itemConfig->setDeletePossible( false );
  _itemConfig->setCategory( _currentCategory, &_categories );
  _stack->raiseWidget( _itemConfig );
  _items.sort();
}

// A slot used by the item config dialog (Delete)
void ShopList::deleteCurrentItem()
{
  _items.remove( _currentEdit );
  delete _currentEdit;
  _items.sort();
}

// A slot used by the item config dialog (OK)
void ShopList::editItemFinished(  const QString& name, CategoryItem* category )
{
  if ( _currentEdit ) {
    _currentEdit->setText( name );
    _currentEdit->setCategory( category );
    _items.sort();
    updateItemView( ); // Just in case we edited the category and it shouldn't be here...
  }
  else {
    addItem( name, category, false, false, "" );
    _items.sort();
    updateItemView( );
  }
}

// Used when you press on an item
void ShopList::editItem( ShoppingItem* item )
{
  _currentEdit = item;

  _itemConfig->setName( _currentEdit->text() );
  _itemConfig->setCategory( _currentEdit->category(), &_categories );
  _itemConfig->setDeletePossible( true );
  _stack->raiseWidget( _itemConfig );
  _items.sort();
}

// Used from the Category Menu (Add)
void ShopList::addCategory()
{
  _editCategory = false;
  _categoryConfig->setName( "" );
  _categoryConfig->setDeletePossible( false );
  _stack->raiseWidget( _categoryConfig );
}

// Used from the Category menu to put up the dialog
void ShopList::editCategory()
{
  _editCategory = true;
  _categoryConfig->setName( _currentCategory->text() );
  _categoryConfig->setDeletePossible( true );
  _stack->raiseWidget( _categoryConfig );
}

// Used from the edit category dialog (OK)
void ShopList::editCategoryFinished(  const QString& name )
{
  CategoryItem *myCategory;
  if ( _editCategory ) {
    _currentCategory->setText( name );
    myCategory = _currentCategory;
  }
  else {
    myCategory = addCategory( name ); // Shouldn't be setting _currentCategory like this
  }
  changeCategory(myCategory);
}

// Generates the Menu dropdown for the categories
void ShopList::populateCategories()
{
  _categoryMenu->clear();

  _showAllAction->addTo( _categoryMenu  );

  for ( QListIterator<CategoryItem> it( _categories ); *it ; ++it ) {
    (*it)->addTo( _categoryMenu );
  }
  _categoryMenu->insertSeparator();
  _addCategoryAction->addTo( _categoryMenu  );
  // Can't edit the Everything category
  if ((! _categories.isEmpty()) &&
      (!_showAllAction->isOn())) _editCategoryAction->addTo( _categoryMenu  );
}

// Used by showNext/PrevCat to simulate selecting a menu from the Category dropdown
// nB The 'setOn()' triggers the selected() signal in the menu above and causes a recursive
// call into changeCategory()
void ShopList::changeCategory( QAction* action)
{
  static bool inFunction = false; // if this doesn't make sense, find out about static local vars
  if (inFunction) return; // this is a horrid recursion protector...
  inFunction = true;
  _currentCategory = static_cast<CategoryItem*>( action );
  if ( action == _showAllAction ) {
    _showAll = true;
    _bar->changeItem(_currCategoryPos, tr("Category: Everything"));
    _showAllAction->setOn(true);
  } else {
    _showAll = false;
    _bar->changeItem(_currCategoryPos, tr ("Category: ")+_currentCategory->text());
    _currentCategory->setOn(true);
  }
  updateItemView();
  inFunction = false;
}

// Circulate throught the categories
void ShopList::showPrevCat()
{
  if (_categories.isEmpty()) return;
  if (_showAll) { // we're at the beginning, go to the end
    changeCategory(_categories.getLast());
    return;
  }
  if (_categories.findRef(_currentCategory) > 0) { // are we at the list start?
    changeCategory(_categories.prev());
  } else {
    changeCategory(_showAllAction);
  }
}
// See above
void ShopList::showNextCat()
{
  if (_categories.isEmpty()) return;
  if (_showAll) {
    changeCategory(_categories.getFirst());
    return;
  }
  if (_categories.findRef(_currentCategory) < (int)_categories.count()-1) {
    changeCategory(_categories.next());
  } else {
    changeCategory(_showAllAction);
  }
}

// Delete Category (removing all associated items first)
void ShopList::deleteCurrentCategory()
{
  // Are you sure?
  if ( QMessageBox::warning(this, tr("Delete Category"), tr("<qt>Deleting the category will delete all the item in the category!</qt>"), tr("Cancel"), tr("Delete")) == 0 )
    return;

  // Remove the items
  ShoppingItem* item;
  for ( ItemListIterator it( _items ); (item = *it); ) {
    ++it;
    if ( item->category() == _currentCategory ) {
      _items.remove( item );
      delete item;
    }
  }

  //
  bool wasSelected = _currentCategory->isOn();
  _categories.remove( _currentCategory );
  delete _currentCategory;
  if ( _categories.count() != 0 ) {
    _currentCategory = _categories.first();
  }
  else { // No categories - disable Setup menu
    menuBar()->setItemEnabled( 1, false ); // Setup Menu
    _editCategoryAction->setEnabled( false );
  }
  if ( wasSelected ) {
    changeCategory(_showAllAction);
  }
}

// The Mode menu activities...

// Always show Everything when switching to this mode
// It's confusing if you're in a Catgory with nothing selected
// because when you switch modes everything disappears
void ShopList::showNeededItems()
{
  _viewNeededMode->setOn(true);
  setCaption( tr("Shopper - Out Shopping (Full list)") );
  _viewMode = ShoppingItem::Needed;
  changeCategory(_showAllAction);
}

// Always show Everything when switching to this mode
// It's confusing if you're in a Catgory with nothing selected
// because when you switch modes everything disappears
void ShopList::showLeftItems()
{
  _viewLeftMode->setOn(true);
  setCaption( tr("Shopper - Out Shopping (What's left)") );
  _viewMode = ShoppingItem::Left;
  changeCategory(_showAllAction);
}

// Don't switch to Everything when switching to this mode
// Most Catgories will have stuff in and the Everything view can be huge.
// In fact, switch to the first() category if looking at Everything
void ShopList::showAllItems()
{
  _viewAllMode->setOn(true);
  setCaption( tr("Shopper - Making a List") );
  _viewMode = ShoppingItem::Making;
  // If we're looking at a particular category then stay there
  // if we're looking at everything then go to the first category
  if (_currentCategory == _showAllAction && !_categories.isEmpty())
    changeCategory(_categories.first());
  else
    updateItemView( );
}

// The biggy - figure out what items to put in the scrolling list...
void ShopList::updateItemView( )
{
  delete _viewManager;
  _viewManager = new QVBoxLayout( _itemView );
  if (_options.isOptionOn("fast.change")) {_itemView->hide();}

  //  _items.sort();
  bool first = true;
  QWidget* last = 0;
  for ( ItemListIterator it( _items ); *it; ++it ) {

    // Show an item if:
    //   Category is right (including 'all')
    // AND
    //   Making a list
    if ( ( _showAll || (*it)->category() == _currentCategory ) &&
	 (( _viewMode == ShoppingItem::Making ) ||
	  ( _viewMode == ShoppingItem::Needed && (*it)->isNeeded() ) ||
	  ( _viewMode == ShoppingItem::Left && (*it)->isNeeded() && !(*it)->isBought() )
	  ) // hmm, should be it->isVisible(_showAll:null?_currentCategory, _viewMode)
	 ) {
      // Don't set it's mode unless we're going to show it
      (*it)->setMode( _viewMode );
      (*it)->show();
      _viewManager->addWidget( *it );

      // Make tabbing etc work
      if ( first ) {
	first = false;
	(*it)->setFocus();
      } else {
	setTabOrder( last, *it );
      }
      last = *it;
    }
    else {
      // It's not suposed to be shown - so...
      (*it)->hide();
    }
  }
  _viewManager->addStretch(1);
  _itemView->show();
}

// Used by the Setup/New Shopping Trip menu
void ShopList::resetItems()
{
  if ( QMessageBox::warning(this, tr("New Shopping Trip?"), tr("<qt>New Shopping Trip means that nothing is marked as 'needed' and all the notes boxes are cleared!</qt>"), tr("Cancel"), tr("Carry On")) == 0 )
    return;

  ShoppingItem* item;
  for ( ItemListIterator it( _items ); (item = *it); ) {
    ++it;
    item->clear();
  }
  // We've just zeroed the list so we want to show everything
  showAllItems();
}
