// Copyright (c) 2002,2003 Jesper Pedersen, David Greaves
//
//     This file is part of Shopper.

//     Shopper is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     Shopper is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with Shopper; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//     $Id: shoplist.h,v 1.3 2003/12/11 21:21:45 arbor Exp $

#ifndef __shoplist
#define __shoplist

#include <qmainwindow.h>
#include <qcheckbox.h>
#include <qlist.h>
#include <qxml.h>
#include <qlistview.h>
#include <qwidgetstack.h>
#include <qaction.h>
#include "shoppingitem.h"
#include "itemconfig.h"
#include "categoryconfig.h"
#include "categoryitem.h"
#include "options.h"
#include "prefsdialog.h"
class QScrollView;
class QCloseEvent;
class QPopupMenu;
class QVBoxLayout;

class ShopList :public QMainWindow, QXmlDefaultHandler
{
Q_OBJECT
public:
  ShopList( QWidget* parent, const char* name = 0 );

protected:
  void load();
  bool startElement( const QString&, const QString&, const QString& qName,
                     const QXmlAttributes& );
  void closeEvent( QCloseEvent* );
  CategoryItem* addCategory( const QString& name );
  void save();
  void updateItemView( );
  void addItem( const QString& name, CategoryItem* category,
		bool needed, bool bought, const QString& note );

protected slots:
  void setPreferences();
  void addNewItem();
  void resetItems();
  void editItem( ShoppingItem* );
  void raiseView();
  void deleteCurrentItem();
  void editItemFinished(  const QString& name, CategoryItem* category );
  void editCategoryFinished(  const QString& name );
  void addCategory();
  void editCategory();
  void populateCategories();
  void changeCategory( QAction* );
  void deleteCurrentCategory();
  void showPrevCat();
  void showNextCat();
  void showNeededItems();
  void showLeftItems();
  void showAllItems();

private:
  ItemList _items;
  CategoryList _categories;
  Options _options;

  QScrollView* _view;
  QWidget* _itemView;
  QVBoxLayout* _viewManager;
  QWidgetStack* _stack;
  QMenuBar* _bar;
  QPopupMenu* _categoryMenu;
  int _currCategoryPos;

  QActionGroup* _categoryGroup;
  QAction* _addCategoryAction;
  QAction* _editCategoryAction;
  QAction* _showAllAction;

  QAction* _viewLeftMode;
  QAction* _viewNeededMode;
  QAction* _viewAllMode;

  // Used at startup for intialisation
  ShoppingItem::viewingMode _startMode;
  CategoryItem* _startCategory;
  CategoryItem* _myCategory;

  CategoryItem* _currentCategory;
  ShoppingItem* _currentEdit;
  bool _editCategory;
  ShoppingItem::viewingMode _viewMode;
  bool _showAll;

  PrefsDialog* _prefsDialog;
  ItemConfig* _itemConfig;  
  CategoryConfig* _categoryConfig;

  void showCat(bool);
  void escape(QString* s);
};



#endif /* shoplist */

