// Copyright (c) 2002,2003 Jesper Pedersen, David Greaves
//
//     This file is part of Shopper.

//     Shopper is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     Shopper is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with Shopper; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//     $Id: shoppingitem.cpp,v 1.2 2003/12/11 16:16:37 arbor Exp $

#include "Shopper.h"
#include "shoppingitem.h"
#include <qevent.h>
#include <qpe/qpeapplication.h>

#include <qlayout.h>

ShoppingItem::ShoppingItem( const QString& name, CategoryItem* category,
			    bool needed, bool bought,  const QString& note,
			    Options *options,
			    QWidget* parent )
  : QHBox(parent, "box", 0, TRUE), _category( category ), _needed( needed ), _bought( bought), _options (options)
{
  _checkB = new ShoppingItemCheckBox(name, this, "item" );
  _checkB->setMinimumWidth(ITEM_WIDTH);
  QHBox *spacer = new QHBox(this);
  _txt = new QLineEdit(note, this, "note" );
  _txt->setMaximumWidth(100);
  spacer->setMinimumWidth(0);
  this->setStretchFactor(spacer, 10);
  
  connect( _checkB, SIGNAL( clicked() ), this, SLOT( stateChange() ) );
  connect( _checkB, SIGNAL( edit() ), this, SLOT( editSlot() ) );
  QPEApplication::setStylusOperation( _checkB, QPEApplication::RightOnHold );
}

// LineEdit note
const QString ShoppingItem::note() {
  return(_txt->text());
}
void ShoppingItem::setNote(const QString &txt) {
  _txt->setText(txt);
}
const QString ShoppingItem::text() {
  return(_checkB->text());
}
void ShoppingItem::setText(const QString &txt) {
  _checkB->setText(txt);
}
void ShoppingItem::setChecked (bool b) {
  _checkB->setChecked (b);
}
const bool ShoppingItem::isChecked() {
  return(_checkB->isChecked());
}


void ShoppingItem::setCategory( CategoryItem* category )
{
  _category = category;
}

CategoryItem* ShoppingItem::category() 
{
  return _category;
}

void ShoppingItem::clear() 
{
  _needed = false;
  _bought = false;
  setNote("");
}

void ShoppingItem::setMode( viewingMode mode ) 
{
  _mode = mode;
  if ( mode == Making ) {
    setChecked( _needed );
  } else {
    setChecked( _bought );
  }
}

void ShoppingItem::stateChange() 
{
  if ( _mode == Making ) {
    _needed = isChecked();
  } else {
    _bought = isChecked();
  }
  // If we're only showing what's left and we're bought - hide
  if ( _mode == Left && _bought == true ) {
    hide();
  }
}

void ShoppingItem::show()
{
  // If we don't hide the checkbox and re-show it we get some v. odd behaviour
  _checkB->hide();
  if (_options->isOptionOn("show.notes")) {
    _txt->show();
  } else {
    _txt->hide();
  }
  _checkB->show();
  QHBox::show();
}

void ShoppingItem::hide() 
{
  QHBox::hide();
}

void ShoppingItem::editSlot()
{
  emit(edit(this));
}

//######################################################################     
int ItemList::compareItems( QCollection::Item i1, QCollection::Item i2 )
{
  ShoppingItem* item1 = reinterpret_cast<ShoppingItem*>(i1);
  ShoppingItem* item2 = reinterpret_cast<ShoppingItem*>(i2);
  
  if ( item1->text() == item2->text() ) 
    return 0;
  else if ( item1->text().lower() > item2->text().lower() ) 
    return 1;
  else
    return -1;
}

//######################################################################
ShoppingItemCheckBox::ShoppingItemCheckBox(const QString & text, QWidget * parent,
					   const char * name=0 ):
  QCheckBox (text,parent,name )
{
}

void ShoppingItemCheckBox::mousePressEvent( QMouseEvent* event ) 
{
  if ( event->button() & RightButton ) {
    emit edit();
  }
  QCheckBox::mousePressEvent( event );
}
