// Copyright (c) 2002,2003 Jesper Pedersen, David Greaves
//
//     This file is part of Shopper.

//     Shopper is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     Shopper is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with Shopper; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//     $Id: shoppingitem.h,v 1.2 2003/12/11 16:16:37 arbor Exp $

#ifndef __SHOPPINGITEM_H
#define __SHOPPINGITEM_H
class CategoryItem;
#include <qlist.h>
#include <qcheckbox.h>
#include <qhbox.h>
#include <qlineedit.h>
#include "options.h"

class ShoppingItemCheckBox :public QCheckBox
{
Q_OBJECT

public:
  ShoppingItemCheckBox (const QString & text, QWidget * parent,
			const char * name=0 );
signals:
  void edit();

protected:
  void mousePressEvent( QMouseEvent* event ) ;

};

class ShoppingItem :public QHBox 
{
Q_OBJECT

public:
  ShoppingItem( const QString& name, CategoryItem* category,
		bool needed, bool bought, const QString& note,
		Options *options,
		QWidget* parent );
  
  // The viewing modes an item can have
  enum viewingMode {Making, Needed, Left};

  // a ShoppingItem 'belongs' to a category
  void setCategory( CategoryItem* category );
  CategoryItem* category();

  // They can be needed or not
  bool isNeeded() const { return _needed; }
  // They can only be 'bought' if they're wanted...
  bool isBought() const { return _bought; }

  // This means it's not wanted or bought
  void clear();

  // The 'mode' determines whether the check box refers to the 'bought'-ness of the
  // item (ie we're out shopping) or the 'needed'-ness (ie we're making a list)
  void setMode( viewingMode );

  // pass through to cbox
  void setChecked (bool);
  const bool isChecked ();
  const QString text ();
  void setText (const QString &);
  // used for the lineedit
  const QString note ();
  void setNote (const QString &);

public slots:
  void hide();
  void show();

signals:  
  void edit( ShoppingItem* );

protected slots:
  void stateChange();
  void editSlot();
 
private:
  CategoryItem* _category;
  bool _needed;
  bool _bought;
  //  State _state;
  viewingMode _mode;
  QCheckBox *_checkB;
  QLineEdit *_txt;
  Options *_options;
};

class ItemList :public QList<ShoppingItem> 
{
protected:
  virtual int compareItems( QCollection::Item i1, QCollection::Item i2 );
};

typedef QListIterator<ShoppingItem> ItemListIterator;

#endif // __SHOPPINGITEM_H
